package nl.voidcorp.discord.commands.management

import nl.voidcorp.discord.command.*
import nl.voidcorp.discord.storage.GuildStore
import org.springframework.stereotype.Service

@Service
class AddBotChannelCommand : Command(
    "addbotchannel",
    aliases = listOf("abc"),
    usage = "addbotchannel #channel (or just the id)",
    commandLevel = CommandLevel.MODERATOR,
    group = CommandGroup.ADMIN,
    location = CommandSource.GUILD
) {
    val regex = "(?:<#?)?(\\d+)>?".toRegex()

    override fun handle(event: CommandMessage): CommandResult {
        val guild = repo.findByGuildId(event.guild!!.idLong) ?: GuildStore(event.guild.idLong)
        if (event.params.drop(1).isEmpty()) {
            val roles = guild.botChannels.map { event.guild.getTextChannelById(it)?.id ?: "Missing channel $it" }
                .joinToString(prefix = "Bot channels: ") { "<#$it>" }
            event.reply(roles)
            return CommandResult.SUCCESS
        }
        val l = mutableListOf<String>()
        for (p in event.params.drop(1)) {
            val res = regex.matchEntire(p)
            if (res != null && res.groupValues.size == 2) {
                if (event.guild.getTextChannelById(res.groupValues[1]) != null) {
                    guild.botChannels.plusAssign(res.groupValues[1].toLong())
                    val channel = event.guild.getTextChannelById(res.groupValues[1])!!
                    l += channel.id
                } else event.reply("There is no channel with id `${res.groupValues[1]}`")
            }
        }
        repo.save(guild)
        if (l.isNotEmpty())
            event.reply(l.joinToString(prefix = "Added the following bot channels: ") { "<#$it>" })

        return CommandResult.SUCCESS
    }
}